# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimForeign,
)


class HemofaxSegmentH(HeaderSegment):

    patients = HprimForeign("P", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True)


class HemofaxSegmentP(Segment):

    orders = HprimForeign("O", "patient")


class HemofaxSegmentO(Segment):

    tube = HprimString(3)
    results = HprimForeign("R", "order")


class HemofaxSegmentR(Segment):

    test = HprimString((3, 4))
    valeur = HprimString(4)
    status = HprimString(9)


class HemofaxSegmentL(Segment):
    pass


class Driver(AstmMessageV2):
    SPEC = {
        "code": "hemofax",
        "label": "Hemofax",
    }
    SEGMENT_CLASS_MAP = {
        "H": HemofaxSegmentH,
        "P": HemofaxSegmentP,
        "O": HemofaxSegmentO,
        "R": HemofaxSegmentR,
        "L": HemofaxSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    ZERO_VALUES = ("0", "0.0", "", "NO")

    def create(self, **kwargs):
        message = HemofaxSegmentH(type_segment="H", **kwargs)
        message.end = HemofaxSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        return self.ORDER

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_o.tube,
                    type="patient",
                )
                # HACK sur CAUTO à remplacer
                sample.results.add(
                    code="LAME",
                    value="HFAXS",
                )
                for seg_r in seg_o.results:

                    if seg_r.status != "F" or seg_r.valeur in self.ZERO_VALUES:
                        continue

                    sample.results.add(
                        code=seg_r.test,
                        value=seg_r.valeur,
                        zero=self.ZERO_VALUES
                    )

        return worklist
